@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: os_info.c,v 1.8 2004/04/06 14:52:56 tdb Exp $
+ * $Id: os_info.c,v 1.9 2004/11/06 18:58:23 tdb Exp $
  */
 
 #include <stdio.h>
@@ -25,7 +25,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-int main(int argc, char **argv){
+int main(){
 
 	sg_host_info *general_stats;
 
