@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: disk_stats.c,v 1.72 2004/07/18 21:30:11 ats Exp $
+ * $Id: disk_stats.c,v 1.79 2004/11/07 15:06:49 ats Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -76,6 +76,20 @@
 			"ntfs"}
 #endif
 
+#ifdef HPUX
+#include <sys/param.h>
+#include <sys/pstat.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/vfs.h>
+#include <mntent.h>
+#include <dirent.h>
+#include <stdio.h>
+#include <time.h>
+#define VALID_FS_TYPES {"vxfs", "hfs"}
+#define DISK_BATCH 30
+#endif
+
 static void disk_stat_init(sg_fs_stats *d) {
 	d->device_name = NULL;
 	d->fs_type = NULL;
@@ -92,7 +106,7 @@ static int is_valid_fs_type(const char *type) {
 	const char *types[] = VALID_FS_TYPES;
 	int i;
 
-	for (i = 0; i < (sizeof types / sizeof *types); i++) {
+	for (i = 0; i < (int) (sizeof types / sizeof *types); i++) {
 		if (strcmp(types[i], type) == 0) {
 			return 1;
 		}
@@ -106,7 +120,7 @@ sg_fs_stats *sg_get_fs_stats(int *entries){
 
 	int valid_type;
 	int num_disks=0;
-#if defined(LINUX) || defined (SOLARIS) || defined(CYGWIN)
+#if defined(LINUX) || defined (SOLARIS) || defined(CYGWIN) || defined(HPUX)
 	FILE *f;
 #endif
 
@@ -116,7 +130,7 @@ sg_fs_stats *sg_get_fs_stats(int *entries){
 	struct mnttab mp;
 	struct statvfs fs;
 #endif
-#if defined(LINUX) || defined(CYGWIN)
+#if defined(LINUX) || defined(CYGWIN) || defined(HPUX)
 	struct mntent *mp;
 	struct statfs fs;
 #endif
@@ -139,8 +153,12 @@ sg_fs_stats *sg_get_fs_stats(int *entries){
 		valid_type = is_valid_fs_type(mp->f_fstypename);
 #endif
 
-#if defined(LINUX) || defined(CYGWIN)
+#if defined(LINUX) || defined(CYGWIN) || defined(HPUX)
+#ifdef MNT_MNTTAB
+	if ((f=setmntent(MNT_MNTTAB, "r" ))==NULL){
+#else
 	if ((f=setmntent("/etc/mtab", "r" ))==NULL){
+#endif
 		sg_set_error(SG_ERROR_SETMNTENT, NULL);
 		return NULL;
 	}
@@ -191,7 +209,7 @@ sg_fs_stats *sg_get_fs_stats(int *entries){
 			/* Freebsd doesn't have a "available" inodes */
 			disk_ptr->used_inodes=disk_ptr->total_inodes-disk_ptr->free_inodes;
 #endif
-#if defined(LINUX) || defined(CYGWIN)
+#if defined(LINUX) || defined(CYGWIN) || defined(HPUX)
 			if (sg_update_string(&disk_ptr->device_name, mp->mnt_fsname) < 0) {
 				return NULL;
 			}
@@ -246,7 +264,7 @@ sg_fs_stats *sg_get_fs_stats(int *entries){
 
 	/* If this fails, there is very little i can do about it, so
 	   I'll ignore it :) */
-#if defined(LINUX) || defined(CYGWIN)
+#if defined(LINUX) || defined(CYGWIN) || defined(HPUX)
 	endmntent(f);
 #endif
 #if defined(SOLARIS)
@@ -295,6 +313,17 @@ sg_disk_io_stats *sg_get_disk_io_stats(int *entries){
 	sg_disk_io_stats *diskio_stats_ptr;
 #endif
 
+#ifdef HPUX
+	long long rbytes = 0, wbytes = 0;
+	struct dirent *dinfo = NULL;
+	struct stat lstatinfo;
+	struct pst_diskinfo pstat_diskinfo[DISK_BATCH];
+	char fullpathbuf[1024] = {0};
+	dev_t diskid;
+	DIR *dh = NULL;
+	int diskidx = 0;
+	int num, i;
+#endif
 #ifdef SOLARIS
 	kstat_ctl_t *kc;
 	kstat_t *ksp;
@@ -342,6 +371,89 @@ sg_disk_io_stats *sg_get_disk_io_stats(int *entries){
 
 	num_diskio=0;
 
+#ifdef HPUX
+	while (1) {
+		num = pstat_getdisk(pstat_diskinfo, sizeof pstat_diskinfo[0],
+		                    DISK_BATCH, diskidx);
+		if (num == -1) {
+			sg_set_error_with_errno(SG_ERROR_PSTAT,
+			                        "pstat_getdisk");
+			return NULL;
+		} else if (num == 0) {
+			break;
+		}
+
+		for (i = 0; i < num; i++) {
+			struct pst_diskinfo *di = &pstat_diskinfo[i];
+
+			/* Skip "disabled" disks. */
+			if (di->psd_status == 0) {
+				continue;
+			}
+	
+			/* We can't seperate the reads from the writes, we'll
+			 * just give the same to each. (This value is in
+			 * 64-byte chunks according to the pstat header file,
+			 * and can wrap to be negative.)
+			 */
+			rbytes = wbytes = ((unsigned long) di->psd_dkwds) * 64LL;
+	
+			/* Skip unused disks. */
+			if (rbytes == 0 && wbytes == 0) {
+				continue;
+			}
+	
+			if (VECTOR_RESIZE(diskio_stats, num_diskio + 1) < 0) {
+				return NULL;
+			}
+	
+			diskio_stats_ptr = diskio_stats + num_diskio;
+	
+			diskio_stats_ptr->read_bytes = rbytes;
+			diskio_stats_ptr->write_bytes = wbytes;
+	
+			diskio_stats_ptr->systime = time(NULL);
+	
+			num_diskio++;
+	
+			/* FIXME This should use a static cache, like the Linux
+			 * code below. */
+			if (diskio_stats_ptr->disk_name == NULL) {
+				dh = opendir("/dev/dsk");
+				if (dh == NULL) {
+					continue;
+				}
+	
+				diskid = (di->psd_dev.psd_major << 24) | di->psd_dev.psd_minor;
+				while (1) {
+					dinfo = readdir(dh);
+					if (dinfo == NULL) {
+						break;
+					}
+					snprintf(fullpathbuf, sizeof(fullpathbuf), "/dev/dsk/%s", dinfo->d_name);
+					if (lstat(fullpathbuf, &lstatinfo) < 0) {
+						continue;
+					}
+	
+					if (lstatinfo.st_rdev == diskid) {
+						if (sg_update_string(&diskio_stats_ptr->disk_name, dinfo->d_name) < 0) {
+							return NULL;
+						}
+						break;
+					}
+				}
+				closedir(dh);
+	
+				if (diskio_stats_ptr->disk_name == NULL) {
+					if (sg_update_string(&diskio_stats_ptr->disk_name, di->psd_hw_path.psh_name) < 0) {
+						return NULL;
+					}
+				}
+			}
+		}
+		diskidx = pstat_diskinfo[num - 1].psd_idx + 1;
+	}
+#endif
 #ifdef OPENBSD
 	mib[0] = CTL_HW;
 	mib[1] = HW_DISKCOUNT;
