@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: network_stats.c,v 1.69 2004/07/18 21:30:11 ats Exp $
+ * $Id: network_stats.c,v 1.71 2004/11/08 08:07:25 ats Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -294,6 +294,10 @@ sg_network_io_stats *sg_get_network_io_stats(int *entries){
 	sg_set_error(SG_ERROR_UNSUPPORTED, "Cygwin");
 	return NULL;
 #endif
+#ifdef HPUX
+	sg_set_error(SG_ERROR_UNSUPPORTED, "HP-UX");
+	return NULL;
+#endif
 
 	*entries=interfaces;
 
@@ -691,6 +695,15 @@ sg_network_iface_stats *sg_get_network_iface_stats(int *entries){
 	close(sock);
 	fclose(f);
 #endif
+#ifdef CYGWIN
+	sg_set_error(SG_ERROR_UNSUPPORTED, "Cygwin");
+	return NULL;
+#endif
+#ifdef HPUX
+	sg_set_error(SG_ERROR_UNSUPPORTED, "HP-UX");
+	return NULL;
+#endif
+
 	*entries = ifaces;
 	return network_iface_stats; 
 }
