@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: os_info.c,v 1.19 2004/07/18 21:30:11 ats Exp $
+ * $Id: os_info.c,v 1.22 2004/11/07 12:31:33 ats Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -46,12 +46,24 @@
 #include <time.h>
 #include <sys/time.h>
 #endif
+#ifdef HPUX
+#include <sys/param.h>
+#include <sys/pstat.h>
+#include <time.h>
+#endif
+
+#include "tools.h"
 
 sg_host_info *sg_get_host_info(){
 
-	static sg_host_info general_stat;	
+	static sg_host_info general_stat;
 	static struct utsname os;
 
+#ifdef HPUX
+	struct pst_static *pstat_static;
+	time_t currtime;
+	long boottime;
+#endif
 #ifdef SOLARIS
 	time_t boottime,curtime;
 	kstat_ctl_t *kc;
@@ -80,6 +92,18 @@ sg_host_info *sg_get_host_info(){
 	general_stat.hostname = os.nodename;
 
 	/* get uptime */
+#ifdef HPUX
+	pstat_static = sg_get_pstat_static();
+	if (pstat_static == NULL) {
+		return NULL;
+	}
+
+	currtime = time(NULL);
+
+	boottime = pstat_static->boot_time;
+
+	general_stat.uptime = currtime - boottime;
+#endif
 #ifdef SOLARIS
 	if ((kc = kstat_open()) == NULL) {
 		sg_set_error(SG_ERROR_KSTAT_OPEN, NULL);
