@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: swap_stats.c,v 1.21 2004/07/18 21:30:11 ats Exp $
+ * $Id: swap_stats.c,v 1.23 2004/11/06 15:41:30 ats Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -53,11 +53,22 @@
 #include <uvm/uvm.h>
 #include <unistd.h>
 #endif
+#ifdef HPUX
+#include <sys/param.h>
+#include <sys/pstat.h>
+#include <unistd.h>
+#define SWAP_BATCH 5
+#endif
 
 sg_swap_stats *sg_get_swap_stats(){
 
 	static sg_swap_stats swap_stat;
 
+#ifdef HPUX
+	struct pst_swapinfo pstat_swapinfo[SWAP_BATCH];
+	int swapidx = 0;
+	int num, i;
+#endif
 #ifdef SOLARIS
 	struct anoninfo ai;
 	int pagesize;
@@ -82,6 +93,43 @@ sg_swap_stats *sg_get_swap_stats(){
 	struct uvmexp *uvm;
 #endif
 
+#ifdef HPUX
+	swap_stat.total = 0;
+	swap_stat.used = 0;
+	swap_stat.free = 0;
+
+	while (1) {
+		num = pstat_getswap(pstat_swapinfo, sizeof pstat_swapinfo[0],
+		                    SWAP_BATCH, swapidx);
+		if (num == -1) {
+			sg_set_error_with_errno(SG_ERROR_PSTAT,
+			                        "pstat_getswap");
+			return NULL;
+		} else if (num == 0) {
+			break;
+		}
+
+		for (i = 0; i < num; i++) {
+			struct pst_swapinfo *si = &pstat_swapinfo[i];
+
+			if ((si->pss_flags & SW_ENABLED) != SW_ENABLED) {
+				continue;
+			}
+	
+			if ((si->pss_flags & SW_BLOCK) == SW_BLOCK) {
+				swap_stat.total += ((long long) si->pss_nblksavail) * 1024LL;
+				swap_stat.used += ((long long) si->pss_nfpgs) * 1024LL;
+				swap_stat.free = swap_stat.total - swap_stat.used;
+			}
+			if ((si->pss_flags & SW_FS) == SW_FS) {
+				swap_stat.total += ((long long) si->pss_limit) * 1024LL;
+				swap_stat.used += ((long long) si->pss_allocated) * 1024LL;
+				swap_stat.free = swap_stat.total - swap_stat.used;
+			}
+		}
+		swapidx = pstat_swapinfo[num - 1].pss_idx + 1;
+	}
+#endif
 #ifdef SOLARIS
 	if((pagesize=sysconf(_SC_PAGESIZE)) == -1){
 		sg_set_error_with_errno(SG_ERROR_SYSCONF, "_SC_PAGESIZE");
