@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: tools.c,v 1.56 2004/07/18 21:30:12 ats Exp $
+ * $Id: tools.c,v 1.62 2004/11/07 12:34:24 ats Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,6 +42,10 @@
 #include <sys/param.h>
 #include <sys/sysctl.h>
 #endif
+#ifdef HPUX
+#include <sys/param.h>
+#include <sys/pstat.h>
+#endif
 
 #include "tools.h"
 #include "statgrab.h"
@@ -230,7 +234,7 @@ static int build_mapping(){
 			if(strcmp(ksp->ks_module, "md")==0) continue;
 			if((kstat_read(kc, ksp, &kios))==-1) continue;
 			strncpy(device_name, ksp->ks_name, sizeof device_name);
-			for(x=0;x<(sizeof device_name);x++){
+			for(x=0;x<(int)(sizeof device_name);x++){
 				if( isdigit((int)device_name[x]) ) break;
 			}
 			if(x == sizeof device_name) x--;
@@ -290,17 +294,7 @@ char *sg_get_string_match(char *line, regmatch_t *match){
 	return match_string;
 }
 
-long long sg_get_ll_match(char *line, regmatch_t *match){
-	char *ptr;
-	long long num;
-
-	ptr=line+match->rm_so;
-	num=atoll(ptr);
-
-	return num;
-}
-#endif
-
+/* Cygwin (without a recent newlib) doesn't have atoll */
 #ifndef HAVE_ATOLL
 static long long atoll(const char *s) {
 	long long value = 0;
@@ -321,6 +315,17 @@ static long long atoll(const char *s) {
 }
 #endif
 
+long long sg_get_ll_match(char *line, regmatch_t *match){
+	char *ptr;
+	long long num;
+
+	ptr=line+match->rm_so;
+	num=atoll(ptr);
+
+	return num;
+}
+#endif
+
 /*      $OpenBSD: strlcpy.c,v 1.8 2003/06/17 21:56:24 millert Exp $     */
 
 /*
@@ -489,6 +494,23 @@ struct uvmexp *sg_get_uvmexp() {
 }
 #endif
 
+#ifdef HPUX
+struct pst_static *sg_get_pstat_static() {
+	static int got = 0;
+	static struct pst_static pst;
+
+	if (!got) {
+		if (pstat_getstatic(&pst, sizeof pst, 1, 0) == -1) {
+			sg_set_error_with_errno(SG_ERROR_PSTAT,
+			                        "pstat_static");
+			return NULL;
+		}
+		got = 1;
+	}
+	return &pst;
+}
+#endif
+
 int sg_init(){
 	sg_set_error(SG_ERROR_NONE, NULL);
 
@@ -513,11 +535,23 @@ int sg_init(){
 }
 
 int sg_drop_privileges() {
+#ifdef HAVE_SETEGID
 	if (setegid(getgid()) != 0) {
+#elif defined(HAVE_SETRESGID)
+	if (setresgid(getgid(), getgid(), getgid()) != 0) {
+#else
+	{
+#endif
 		sg_set_error_with_errno(SG_ERROR_SETEGID, NULL);
 		return -1;
 	}
+#ifdef HAVE_SETEUID
 	if (seteuid(getuid()) != 0) {
+#elif defined(HAVE_SETRESUID)
+	if (setresuid(getuid(), getuid(), getuid()) != 0) {
+#else
+	{
+#endif
 		sg_set_error_with_errno(SG_ERROR_SETEUID, NULL);
 		return -1;
 	}
@@ -532,4 +566,3 @@ void *sg_realloc(void *ptr, size_t size) {
 	}
 	return tmp;
 }
-
