@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: saidar.c,v 1.29 2004/04/07 15:50:26 tdb Exp $
+ * $Id: saidar.c,v 1.32 2004/11/07 12:36:04 ats Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -377,7 +377,7 @@ void display_data(){
 			move(line, 62);
 			printw("%7s", size_conv(disk_stat_ptr->avail));
 			move(line, 73);
-			printw("%5.2f%%", 100.00 * ((float) disk_stat_ptr->used / (float) (disk_stat_ptr->used + disk_stat_ptr->avail)));
+			printw("%6.2f%%", 100.00 * ((float) disk_stat_ptr->used / (float) (disk_stat_ptr->used + disk_stat_ptr->avail)));
 			disk_stat_ptr++;
 			line++;
 		}
@@ -386,7 +386,7 @@ void display_data(){
 	refresh();
 }
 
-void sig_winch_handler(int sig){
+void sig_winch_handler(int dummy){
 	clear();
 	display_headings();
 	display_data();
